

// Copyright: Hans de Nivelle, May 2005.
//
// An element is an element of a model.


// This file is part of Geo2007F.
//
// Geo 2007F is free software: You can distribute it and/or modify
// it under the terms of the GNU General Public License, Version 3,
// as published by the Free Software Foundation.
//
// Geo2007F is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License 
// Version 3 along with Geo2007F. It not, see 
// < http://www.gnu.org/licenses/ >



#ifndef ELEMENT_INCLUDED 
#define ELEMENT_INCLUDED  1


#include <iostream>


#include "assert.h"


class element
{
   unsigned int x;

private:
   explicit element( unsigned int x )
      : x (x) 
   {
   }
 
public: 
   element( ) : x(0) { }

   element operator ++ ( int );
   element& operator ++ ( );
   element operator -- ( int );
   element& operator -- ( ); 
  
   unsigned int getunsigned( ) const; 
   static element constructfromunsigned( unsigned int i );

   friend bool operator == ( element, element );
   friend bool operator != ( element, element );
   friend bool operator < ( element, element );
   friend bool operator > ( element, element );
   friend bool operator <= ( element, element );
   friend bool operator >= ( element, element );

   
   friend std::ostream& operator << ( std::ostream&, element ); 
};


inline element element::operator ++ ( int ) 
{
   element res = *this;
   x ++ ;
   return res;
}

inline element element::operator -- ( int )
{
   ASSERT( x != 0 );
   element res = *this;
   x -- ;
   return res;
}

inline element& element::operator ++ ( ) 
{
   x ++ ;
   return *this;
}

inline element& element::operator -- ( )
{
   ASSERT( x != 0 );
   x -- ;
   return *this;
}


inline unsigned int element::getunsigned( ) const
{
   return x;
}

inline element element::constructfromunsigned( unsigned int i )
{
   return element(i);
}


inline bool operator == ( element e1, element e2 )
{
   return ( e1. x == e2. x );
}

inline bool operator != ( element e1, element e2 )
{
   return ( e1. x != e2. x );
}

inline bool operator < ( element e1, element e2 )
{
   return ( e1. x < e2. x );
}

inline bool operator > ( element e1, element e2 )
{
   return ( e1. x > e2. x );
}

inline bool operator <= ( element e1, element e2 )
{
   return ( e1. x <= e2. x );
}

inline bool operator >= ( element e1, element e2 )
{
   return ( e1. x >= e2. x );
}



inline std::ostream& operator << ( std::ostream& stream, element e )
{
   stream << "E" << e.x;
   return stream;
}


#endif


