
// Copyright: Hans de Nivelle,
// November 2005.


// This file is part of Geo2007F.
//
// Geo 2007F is free software: You can distribute it and/or modify
// it under the terms of the GNU General Public License, Version 3,
// as published by the Free Software Foundation.
//
// Geo2007F is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License 
// Version 3 along with Geo2007F. It not, see 
// < http://www.gnu.org/licenses/ >


// A partial object is an object that does or does not exist.



#ifndef PARTIAL_INCLUDED
#define PARTIAL_INCLUDED   1

#include <iostream> 
#include <vector>

#include "assert.h"


template< class X > class partial
{

private:
   std::vector< X > repr;

public:
   partial( const X& val )
   {
      repr. push_back( val );
   }

   partial( )
   {
   }


   bool isdefined( ) const
   {
      return repr. size( ) == 1;
   }

   bool isundefined( ) const
   {
      return repr. size( ) == 0;
   }

   void clear( )
   {
      repr. clear( );
   }

   void operator = ( const partial< X > & other )
   {
      repr = other. repr;
   }

   void operator = ( const X& other )
   {
      if( repr. size( ) == 0 )
         repr. push_back( other );
      else
         repr [0] = other;
   }


   const X& contents( ) const
   {
      ASSERT( repr. size( ));
      return repr [0];
   }

   X& contents( ) 
   {
      ASSERT( repr. size( ));
      return repr [0];
   }

};


template< class X >
std::ostream& operator << ( std::ostream& stream, const partial<X> & p )
{
   if( p. isundefined( ))
      stream << "UNDEFINED";
   else
      stream << p. contents( );

   return stream;
}


#endif

