

// Copyright: Hans de Nivelle, June 2006.
// Modified for Bliksem, september 2006.


// This file is part of Geo2007F.
//
// Geo 2007F is free software: You can distribute it and/or modify
// it under the terms of the GNU General Public License, Version 3,
// as published by the Free Software Foundation.
//
// Geo2007F is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License 
// Version 3 along with Geo2007F. It not, see 
// < http://www.gnu.org/licenses/ >



#ifndef PROLOG_EXPRESSION_INCLUDED
#define PROLOG_EXPRESSION_INCLUDED   1


#include "funcpred.h"

#include <iostream> 
#include <list>
#include <vector>



namespace prolog 
{


   class expression
   {

      struct expression_node
      {
         unsigned int reference_count;
         funcpred func; 
         std::list< expression > subexpressions;


	 expression_node( funcpred func, 
			  const std::list< expression > & subexpressions )
            : func( func ),
	      subexpressions( subexpressions )
         {
         }


	 expression_node( funcpred func, 
			  const std::vector< expression > & subexpressions )
            : func( func ) 
         {
            for( std::vector< expression > :: const_iterator 
			 p = subexpressions. begin( );
			 p != subexpressions. end( );
			 ++ p )
            {
               ( this -> subexpressions ). push_back( *p );
            }
         }

      };


   private:
      expression_node* repr;




   public: 
      expression( const expression& other );
      void operator = ( expression other );
      ~expression( );

      expression( funcpred f, 
		  const std::list< expression > & subexpressions );
      expression( funcpred f, 
		  const std::vector< expression > & subexpressions );
         // The number of subexpressions must fit to the arity of f.

      std::list< expression > :: const_iterator subexpressions_begin( ) const;
      std::list< expression > :: const_iterator subexpressions_end( ) const;

      funcpred getconstructor( ) const;
      unsigned int getnrsubexpressions( ) const;
         // Equal to getfunction( ). getarity( ) of course.

   };

   std::ostream& operator << ( std::ostream& stream, const expression& exp );

}


#endif


