

#ifndef PROLOG_SYNTACTICDEFSFORIDENTIFIER_INCLUDED
#define PROLOG_SYNTACTICDEFSFORIDENTIFIER_INCLUDED  1


// Copyright: Ravinder Indarapu, modified by Hans de Nivelle,
// in March 2003, and one more time modified in september 2006. 


// This file is part of Geo2007F.
//
// Geo 2007F is free software: You can distribute it and/or modify
// it under the terms of the GNU General Public License, Version 3,
// as published by the Free Software Foundation.
//
// Geo2007F is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License 
// Version 3 along with Geo2007F. It not, see 
// < http://www.gnu.org/licenses/ >



#include "../assert.h" 
#include "syntacticdef.h"


namespace prolog 
{

   class syntacticdefsforidentifier
   {

   private: 

      const syntacticdef* prefixdef;
      const syntacticdef* infixdef;
      const syntacticdef* postfixdef;

   public: 


      syntacticdefsforidentifier( ); 
      syntacticdefsforidentifier( const syntacticdefsforidentifier& other );
      ~syntacticdefsforidentifier( );

 
// Insert toadd into this syntacticdefsforidentifier. If toadd 
// is a prefixoperator definition, and there already is a 
// prefixoporator defined, then the existing definition of overwritten,
// insert returns true. If there is no existing value, then insert
// returns false. 
// The same applies to inserting of an infixoperator or postfixoperator. 

      bool insert( const syntacticdef& toadd );


// Remove toremove if it occurs in this syntacticdefsforidentifier.
// Otherwise do nothing. In case something was removed,
// we return true. 

      bool remove( const syntacticdef& toremove );


      void operator = ( const syntacticdefsforidentifier& other ); 


      bool hasprefixdef( ) const;
      bool hasinfixdef( ) const;
      bool haspostfixdef( ) const;


      const syntacticdef& getprefixdef( ) const;
      const syntacticdef& getinfixdef( ) const;
      const syntacticdef& getpostfixdef( ) const;


   private:
      const syntacticdef* makecopy( const syntacticdef* ); 
         /* Makes a copy if the points is different from 0. */ 

   };

   std::ostream& operator << ( std::ostream&, 
                               const syntacticdefsforidentifier& ); 
}


#endif



