

// Copyright: Hans de Nivelle, April 2006.


// This file is part of Geo2007F.
//
// Geo 2007F is free software: You can distribute it and/or modify
// it under the terms of the GNU General Public License, Version 3,
// as published by the Free Software Foundation.
//
// Geo2007F is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License 
// Version 3 along with Geo2007F. It not, see 
// < http://www.gnu.org/licenses/ >


#include "occurrencematrix.h"


namespace swll
{

   const std::vector< occurrencematrix::within_clause > & 
   occurrencematrix::row::operator [] ( unsigned int x ) const
   {
      ASSERT( x < repr. size( ));
      return repr [x];
   }


   std::vector< occurrencematrix::within_clause > & 
   occurrencematrix::row::operator [] ( unsigned int x ) 
   {
      if( x >= repr. size( ))
      {
         repr. insert( repr. end( ), 1 + x - repr. size( ),
                       std::vector< within_clause > ( )); 
      }
      return repr [x];
   }




   const occurrencematrix::row& 
   occurrencematrix::operator [] ( variable v ) const
   {
      unsigned int k = v. getunsigned( );
      ASSERT( k < repr. size( ));
      return repr [k];
   }


   occurrencematrix::row& 
   occurrencematrix::operator [] ( variable v ) 
   {
      unsigned int k = v. getunsigned( );
      if( k >= repr. size( ))
      {
         repr. insert( repr. end( ), 1 + k - repr. size( ), row( ) ); 
      }
      return repr [k];
   }

 
   void occurrencematrix::clear( )
   {
      repr. clear( );
   }


   void occurrencematrix::insertoccurrence(
                std::vector< within_clause > & w,
		unsigned int clausenr, unsigned int offset )
   {
      for( std::vector< within_clause > :: iterator
              p = w. begin( );
	      p != w. end( );
	      ++ p )
      {
         if( p -> clausenr == clausenr )
         {
            p -> offsets. push_back( offset );
	    return;
         }
      }
      w. push_back( within_clause( clausenr ));
      w. back( ). offsets. push_back( offset );
   }


   std::ostream& operator << ( std::ostream& stream,
		               const occurrencematrix& m ) 
   {
      stream << "occurrence matrix:\n";

      for( unsigned int v = 0; v < m. repr. size( ); ++ v )
      {
         for( unsigned int x = 0; x < m. repr [v]. repr. size( ); ++ x )
         {
            const std::vector< occurrencematrix::within_clause > & 
	       occ = m. repr [v]. repr [x];

	    if( occ. size( ))
            {
	       stream << "assignment( ";
	       stream << "( " << variable::constructfromunsigned(v);
	       stream << " := ";
	       stream << x << " ): ";

	       stream << "[ "; 
	       for( std::vector< occurrencematrix::within_clause > :: 
	                     const_iterator
                       p = occ. begin( );
		       p != occ. end( );
		       ++ p )
               {
                  if( p != occ. begin( ))
                     stream << ", "; 

		  stream << p -> clausenr << ":[";
		  for( std::list< unsigned int > :: const_iterator
                          q = p -> offsets. begin( );
			  q != p -> offsets. end( );
			  ++ q )
                  {
                     if( q != p -> offsets. begin( ))
                        stream << ",";
		     stream << *q;
                  }
		  stream << "]"; 
               }
	       stream << " ]\n"; 
            }
         }
      }
   
      return stream; 
   }
} 



