

// Copyright: Hans de Nivelle, May 2005.


// This file is part of Geo2007F.
//
// Geo 2007F is free software: You can distribute it and/or modify
// it under the terms of the GNU General Public License, Version 3,
// as published by the Free Software Foundation.
//
// Geo2007F is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License 
// Version 3 along with Geo2007F. It not, see 
// < http://www.gnu.org/licenses/ >


#ifndef VARIABLE_INCLUDED 
#define VARIABLE_INCLUDED  1


#include <iostream>


#include "assert.h"


class variable
{
   unsigned int x;

private:
   explicit variable( unsigned int x )
      : x (x) 
   {
   }
 
public: 
   variable( ) : x(0) { }

   variable operator ++ ( int );
   variable& operator ++ ( );
   variable operator -- ( int );
   variable& operator -- ( ); 
  
   unsigned int getunsigned( ) const; 
   static variable constructfromunsigned( unsigned int i );

   friend bool operator == ( variable, variable );
   friend bool operator != ( variable, variable );
   friend bool operator < ( variable, variable );
   friend bool operator > ( variable, variable );
   friend bool operator <= ( variable, variable );
   friend bool operator >= ( variable, variable );

   
   friend std::ostream& operator << ( std::ostream&, variable ); 
};


inline variable variable::operator ++ ( int ) 
{
   variable res = *this;
   x ++ ;
   return res;
}

inline variable variable::operator -- ( int )
{
   ASSERT( x != 0 );
   variable res = *this;
   x -- ;
   return res;
}

inline variable& variable::operator ++ ( ) 
{
   x ++ ;
   return *this;
}

inline variable& variable::operator -- ( )
{
   ASSERT( x != 0 );
   x -- ;
   return *this;
}


inline unsigned int variable::getunsigned( ) const
{
   return x;
}

inline variable variable::constructfromunsigned( unsigned int i )
{
   return variable(i);
}


inline bool operator == ( variable v1, variable v2 )
{
   return ( v1. x == v2. x );
}

inline bool operator != ( variable v1, variable v2 )
{
   return ( v1. x != v2. x );
}

inline bool operator < ( variable v1, variable v2 )
{
   return ( v1. x < v2. x );
}

inline bool operator > ( variable v1, variable v2 )
{
   return ( v1. x > v2. x );
}

inline bool operator <= ( variable v1, variable v2 )
{
   return ( v1. x <= v2. x );
}

inline bool operator >= ( variable v1, variable v2 )
{
   return ( v1. x >= v2. x );
}



inline std::ostream& operator << ( std::ostream& stream, variable v )
{
   stream << "V" << v. x;
   return stream;
}


#endif


