

// Copyright: Hans de Nivelle, Jan 2006.
// Added const_iterator in February 2007. 


// This file is part of Geo2007F.
//
// Geo 2007F is free software: You can distribute it and/or modify
// it under the terms of the GNU General Public License, Version 3,
// as published by the Free Software Foundation.
//
// Geo2007F is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License 
// Version 3 along with Geo2007F. It not, see 
// < http://www.gnu.org/licenses/ >



#ifndef VARSET_INCLUDED
#define VARSET_INCLUDED  1


#include <vector>
#include <iostream>


#include "variable.h"
#include "varatom.h"



class varset 
{

   std::vector< variable > repr;

public: 
   bool contains( variable v ) const;
   unsigned int size( ) const;
   void clear( ); 

   void insert( variable v );
   void remove( variable v );

   variable supremum( ) const; 


   typedef std::vector< variable > :: const_iterator const_iterator;

   const_iterator vars_begin( ) const { return repr. begin( ); } 
   const_iterator vars_end( ) const { return repr. end( ); } 


   friend varset operator | ( const varset& , const varset& );
   friend varset operator & ( const varset& , const varset& );
   friend varset operator - ( const varset& , const varset& );

};


varset operator | ( const varset& v1, const varset& v2 );
varset operator & ( const varset& v1, const varset& v2 );
varset operator - ( const varset& v1, const varset& v2 );

std::ostream& operator << ( std::ostream& stream, const varset& vs );

#endif


